`include "vme64x_bfm.svh"
`include "svec_vme_buffers.svh"

import wishbone_pkg::*;
   
module main;
   reg rst_n = 0;
   reg clk_125m_pllref = 0;
   reg clk_62m5_pllref = 0;
   wire clk_62m5;
   wire rst_62m5_n;

   reg 	clk_local = 0;

   always #25ns clk_local <= ~clk_local;
   
   
   initial begin
      repeat(20) @(posedge clk_125m_pllref);
      rst_n = 1;
   end

   always@(posedge clk_125m_pllref)
     clk_62m5_pllref <= ~clk_62m5_pllref;
   
   
   // 125Mhz
   always #4ns clk_125m_pllref <= ~clk_125m_pllref;

   IVME64X VME(rst_n);

   `DECLARE_VME_BUFFERS(VME.slave);

   logic             ddr_reset_n;
   logic             ddr_ck_p;
   logic             ddr_ck_n;
   logic             ddr_cke;
   logic             ddr_ras_n;
   logic             ddr_cas_n;
   logic             ddr_we_n;
   wire  [7:0]       ddr_dm;
   logic [2:0]       ddr_ba;
   logic [14:0]      ddr_a;
   wire  [63:0]      ddr_dq;
   wire  [7:0]       ddr_dqs_p;
   wire  [7:0]       ddr_dqs_n;
   wire              ddr_rzq;
   logic             ddr_odt;

   logic [4:0]       slot_id = 5'h8;

   wire [4:0] 	     afpga_tx, afpga_rx;
   wire 	     afpga_clk_p, afpga_clk_n;
   
   
/* -----\/----- EXCLUDED -----\/-----
   svec7_sfpga_top
     #(
       .g_SIMULATION(1'b1)
       )
   DUT (
	.rst_n_i(rst_n),
        .lclk_i (clk_local),

	.afpga_clk_p_i(afpga_clk_p),
	.afpga_clk_n_i(afpga_clk_n),
	.afpga_tx_o(afpga_rx),
	.afpga_rx_i(afpga_tx),
        .vme_as_n_i               (VME_AS_n),
        .vme_sysreset_n_i         (VME_RST_n),
        .vme_write_n_i            (VME_WRITE_n),
        .vme_am_i                 (VME_AM),
        .vme_ds_n_i               (VME_DS_n),
        .vme_gap_i                (^slot_id),
        .vme_ga_i                 (~slot_id),
        .vme_berr_o               (VME_BERR),
        .vme_dtack_n_o            (VME_DTACK_n),
        .vme_retry_n_o            (VME_RETRY_n),
        .vme_retry_oe_o           (VME_RETRY_OE),
        .vme_lword_n_b            (VME_LWORD_n),
        .vme_addr_b               (VME_ADDR),
        .vme_data_b               (VME_DATA),
        .vme_irq_o                (VME_IRQ_n),
        .vme_iack_n_i             (VME_IACK_n),
        .vme_iackin_n_i           (VME_IACKIN_n),
        .vme_iackout_n_o          (VME_IACKOUT_n),
        .vme_dtack_oe_o           (VME_DTACK_OE),
        .vme_data_dir_o           (VME_DATA_DIR),
        .vme_data_oe_n_o          (VME_DATA_OE_N),
        .vme_addr_dir_o           (VME_ADDR_DIR),
        .vme_addr_oe_n_o          (VME_ADDR_OE_N)
      
      
  	);
 -----/\----- EXCLUDED -----/\----- */

      svec7_test_top
     #(
       .g_SIMULATION(1'b1)
       )
      DUT2(
	   .rst_n_i(rst_n),

         .clk_125m_pllref_p_i      (clk_125m_pllref),
         .clk_125m_pllref_n_i      (~clk_125m_pllref),

         .clk_20m_vcxo_i (clk_local),
         .clk_125m_gtx_n_i (clk_125m_pllref),
         .clk_125m_gtx_p_i (~clk_125m_pllref),

	   .clk_fpga2_p_i(clk_125m_pllref),
	   .clk_fpga2_n_i(~clk_125m_pllref),
	   .clk_si57x_p_i(clk_125m_pllref),
	   .clk_si57x_n_i(~clk_125m_pllref),
	   

 	   .sfpga_clk_p_o(afpga_clk_p),
	   .sfpga_clk_n_o(afpga_clk_n),
	   .sfpga_tx_o(afpga_tx),
	   .sfpga_rx_i(afpga_rx),
   
	 .vme_sysreset_n_i(VME_RST_n),
	   
         .fmc0_scl_b (),
         .fmc0_sda_b (),
         .fmc1_scl_b (),
         .fmc1_sda_b (),
       

         .onewire_b (),

         .carrier_scl_b (),
         .carrier_sda_b (),

   
         .uart_rxd_i (),
         .uart_txd_o (),

         .pll20dac_din_o (),
         .pll20dac_sclk_o (),
         .pll20dac_sync_n_o (),
         .pll25dac_din_o (),
         .pll25dac_sclk_o (),
         .pll25dac_sync_n_o (),

         .sfp_txp_o (),
         .sfp_txn_o (),
         .sfp_rxp_i (),
         .sfp_rxn_i (),
       
         .sfp_mod_def1_b (),
         .sfp_mod_def2_b (),
        

	  .fp_gpio3_b(1'b0),
	  .fp_gpio4_b(1'b0)
	   
     
  	  );


   task automatic init_vme64x_core(ref CBusAccessor_VME64x acc);
      uint64_t rv;

      /* map func0 to 0x80000000, A32 */

      acc.write('h7ff63, 'h80, A32|CR_CSR|D08Byte3);
      acc.write('h7ff67, 0, CR_CSR|A32|D08Byte3);
      acc.write('h7ff6b, 0, CR_CSR|A32|D08Byte3);
      acc.write('h7ff6f, 32, CR_CSR|A32|D08Byte3);
      acc.write('h7ff33, 1, CR_CSR|A32|D08Byte3);
      acc.write('h7fffb, 'h10, CR_CSR|A32|D08Byte3); /* enable module (BIT_SET = 0x10) */

      acc.read(      'h7ff63, rv, CR_CSR|A32|D08Byte3);

      $display("Rv %x", rv );

      acc.set_default_modifiers(A32 | D32 | SINGLE);

   endtask // init_vme64x_core
     
   
   initial begin
      uint64_t d;
      uint64_t addr[1];
      uint64_t data[] = new[512];
      real t_start, t_end;
      
      
      int i, result;
      
      automatic CBusAccessor_VME64x acc = new(VME.tb);
//      automatic CWishboneAccessor ddr4_acc = xwb_ddr4.get_accessor();

      #5us;
      init_vme64x_core(acc);

      //  Display meta data
      d = 'hdeadbeef;
      acc.write('h80000000, d, A24|SINGLE|D32);
      d = 'hcafebabe;
      acc.write('h80000004, d, A24|SINGLE|D32);

      acc.read('h80000000, d, A24|SINGLE|D32);
      $display("Rdbk[0] = %x", d);
      
      acc.read('h80000004, d, A24|SINGLE|D32);
      $display("Rdbk[4] = %x", d);
      //$display("ddr status: %x", d);

      addr[0] = 'h80001000;

      
      t_start = real'($time) / real'( 1us );
      acc.readm( addr, data,  A32|MBLT|D32, result );
      t_end = real'($time) / real'(1us);

      $display("mblt : %d bytes / %.0f us = approx %.0f bytes / second", data.size() * 4, t_end-t_start, real'(data.size()) * 4 * 1e6/(t_end-t_start));
      
      
      
   end

  
endmodule // main



